/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

////////////////////////////////////////////////////////////////////////
//
// ScratchWindow.h
//	
//	A window for rendering OpenGL.  This class has no relation to the
//	dmu library.  It is just some sample code to show how to open
//	and set up a double buffering window for OpenGL rendering.
//	The following is an example of how this class is used.
//
//      ScratchWindow *window;
//      window = ScratchWindow::create(theDisplay, sizeX, sizeY);
//      if (!window){
//         fprintf (stderr, "Failed to create scratch window!\n");
//         exit (-1);
//      }
//      window->setContext();
//
//      glViewport( 0, 0, sizeX, sizeY);
//      glMatrixMode(GL_PROJECTION);
//      glLoadIdentity();
//      glOrtho(0,sizeX,0,sizeY,0,1000);
//      . . . continue Open GL here . . 
//
////////////////////////////////////////////////////////////////////////

#ifndef _ScratchWindow_H
#define _ScratchWindow_H

#include <sys/types.h>		// size_t

#include <GL/glx.h>

#include <Xm/Xm.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

class ScratchWindow
{
public:
    // The create function returns NULL if the window could not be
    // created.  The window is guaranteed to have 8 bits each of red,
    // greed, blue, and alpha.
    static ScratchWindow* create( Display*, int width, int height,int *argc, String *argv );
    virtual ~ScratchWindow();
    
    // setContext() makes this window the current GL context.  The
    // coordinate system is set up so that the rectangle from 
    // (0,0,0) to (width-1,height-1,0) will fill the window.
    virtual void   setContext()		= 0;

    virtual int    getWidth() 		= 0;
    virtual int    getHeight() 		= 0;
    virtual size_t getBufferSize()	= 0;
    virtual GLXContext getContext()     = 0;
    virtual void   addGLCallback(String name,
			       XtCallbackProc func,
			       XtPointer data) = 0;
    virtual void   eventLoop() = 0;
    virtual void addInput(int fd,XtInputCallbackProc func,XtPointer data) = 0;

};

#endif // _ScratchWindow_H
