/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmu.h
//
//  Author: Ed Allard & Tim Davison
//  Date:   Oct, 1996
//
#ifndef _DMU_H
#define _DMU_H

// some standard includes
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>

// define TRUE and FALSE
#define DMU_TRUE   1
#define DMU_FALSE  0

#define DMU_PROPAGATE     1
#define DMU_NO_PROPAGATE  0


// Return values
#define DMU_OK              0
#define DMU_GENERIC_ERROR   1
#define DMU_UNIMPLEMENTED   2
#define DMU_VIRTUAL_CALLED  3
#define DMU_BAD_CONSTRAINTS 4
#define DMU_PARAMS_CREATE_FAILED       5
#define DMU_POOL_DEFAULTS_FAILED       6
#define DMU_POOL_CREATE_FAILED         7
#define DMU_BAD_INPUT                  8
#define DMU_NOT_MY_EVENT               9
#define DMU_NO_RUNTIME_ADD             10
#define DMU_NOT_STARTED                11

// Video Return Values
#define DMU_VIDEO_ERROR               1000
#define DMU_VIDEO_GETNODE_FAILED      1001
#define DMU_VIDEO_CREATEPATH_FAILED   1002
#define DMU_VIDEO_SETUPPATH_FAILED    1003
#define DMU_VIDEO_BAD_CONTROL_VALUE   1004
#define DMU_VIDEO_BUF_REGISTER_FAILED 1005
#define DMU_VIDEO_START_FAILED        1006
#define DMU_VIDEO_STARTED             1007
#define DMU_VIDEO_NO_EVENT            1008
#define DMU_VIDEO_TO_DMBUFFER_FAILED  1009
#define DMU_VIDEO_NO_PATH             1010
// Temporary Return values - see handleEvents docs in the header for more 
#define VIDEOIN_TRANSFER_COMPLETE    1011
#define VIDEOIN_RANDOM_EVENT         1012
#define VIDEOIN_SHUTTER_EVENT        1013

// dmIC return values
#define DMU_DMIC_NO_IMG_CTX_DEFINED          2000
#define DMU_DMIC_SRC_PARAMS_FAILED           2001
#define DMU_DMIC_DST_PARAMS_FAILED           2002
#define DMU_DMIC_CONV_PARAMS_FAILED          2003
#define DMU_DMIC_INVALID_QUALITY             2004
#define DMU_DMIC_SEND_FAILED                 2005
#define DMU_DMIC_RECEIVE_FAILED              2006
#define DMU_DMIC_SET_CONV_FAILED             2007
#define DMU_DMIC_CREATE_FAILED               2008
#define DMU_DMIC_FOUND_NO_USABLE_DMIC        2009
#define DMU_DMIC_SRC_POOL_PARAMS_FAILED      2010
#define DMU_DMIC_DST_POOL_PARAMS_FAILED      2011
#define DMU_DMIC_DST_POOL_FAILED             2012
#define DMU_DMIC_SET_IMAGE_DEFAULTS_FAILED   2013
#define DMU_DMIC_SET_POOL_DEFAULTS_FAILED    2014
#define DMU_DMIC_PARAMS_SET_STRING_FAILED    2015
#define DMU_DMIC_PARAMS_SET_ENUM_FAILED      2016
#define DMU_DMIC_BUFFER_FREE_FAILED          2017

// GL related return values
#define DMU_GL_ERROR         3000
#define DMU_GL_NO_FBCONFIG   3001
#define DMU_GL_PBUFFER_ALLOC_FAILED    3002
#define DMU_GL_PBUFFER_SETUP_FAILED    3003
#define DMU_GL_PBUFFER_ASSOC_FAILED    3004

// Movie related return values
#define DMU_MOVIE_ERROR        4000
#define DMU_MOVIE_THREAD_ERROR 4001
#define DMU_MOVIE_FILE_ERROR   4002
#define DMU_MOVIE_SEMA_ERROR   4004
#define DMU_MOVIE_FRAME_REPEAT 4005  // Asked to read the same frame twice
#define DMU_MOVIE_BAD_BUFFER_SIZE 4006  
#define DMU_MOVIE_BAD_OFFSET 4007

// Chain related return values 
#define DMU_CHAIN_ERROR         5000
#define DMU_CHAIN_NO_CHAIN      5001
#define DMU_CHAIN_USE_SET_FD    5002

// Pixel trans Errors
#define DMU_PIXEL_ERROR         6000

// Network Errors
#define DMU_NETWORK_ERROR       7000

// These are paramaters to set on the DMbuffer pool
// They deal with whether or not the cpu uses the cache when it load
// data from these buffers.  This can be a very important performance
// decision.  
#define	DMU_NOT_MAPPED	DM_FALSE
#define	DMU_NOT_CACHED	DM_FALSE
#define	DMU_MAPPED      DM_TRUE
#define	DMU_CACHED	DM_TRUE

#define DMU_SOFTWARE_COMPRESSION   0
#define DMU_HW_JPEG_COMPRESSION    1

// for debugging purposes
#define DEBUG(x)   x
#define DEBUG1(x) 
#define DEBUG2(x)
#define DEBUG3(x) 

#define dmuerr(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"Error: file:%s on line:%d\n", __FILE__, __LINE__); \
       fprintf(stderr,"Error:%d:%s\n", dmuErrID, getErrorString(dmuErrID) ); \
    }\
 }

#endif // _DMU_H
