/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuError.h
//
//  Author: Ed Allard & Tim Davison
//  Date:   Oct, 1996
//
//  This file contains the error messages associated with
//  the error symbols defined in dmu.h
//  in dmuObj.c++ there is a function that will retreive
//  an error message given the error number.  There is
//  also support for making error fatal, so that if you
//  look up the number and it's a fatal number, you exit
//  immediately
//
#ifndef _DMUERROR_H
#define _DMUERROR_H

// error structure
typedef struct _errorMsg {
  int errorID;
  char *errorString;
  int fatal;
} errorMsg;

const char *noErrorMsg = "No error message matches that errorID";
const errorMsg dmuErrors[] = {
  // Generic Errors
  {DMU_OK, "No Error", DMU_FALSE },
  {DMU_GENERIC_ERROR, "Generic error", DMU_FALSE},
  {DMU_UNIMPLEMENTED, "Feature not implemented (yet)", DMU_FALSE},
  {DMU_VIRTUAL_CALLED,
   "Virtual method called.  Should be overriden", DMU_TRUE}, 
  {DMU_BAD_CONSTRAINTS,
   "Can't meet constraints to create DMbuffer Pool", DMU_FALSE},
  {DMU_PARAMS_CREATE_FAILED,  
   "Uable to create pool parameter object",DMU_TRUE},
  {DMU_POOL_DEFAULTS_FAILED,  
   "Unable to set pool parameter defaults",DMU_FALSE},
  {DMU_POOL_CREATE_FAILED, 
   "Unable to create buffer pool", DMU_TRUE},
  {DMU_BAD_INPUT,
   "The params passed to function are not valid - possibly NULL?", DMU_FALSE},
  {DMU_NOT_MY_EVENT,
   "HandleEvents called but fd not set - this is not neccessarily an error",
   DMU_FALSE},
  {DMU_NO_RUNTIME_ADD, "Can't add child at runtime", DMU_FALSE },
  
  // Video errors
  {DMU_VIDEO_ERROR, "Generic video failure",DMU_FALSE},
  {DMU_VIDEO_GETNODE_FAILED, "Unable to get video node",DMU_FALSE},
  {DMU_VIDEO_CREATEPATH_FAILED, "Unable to create video path",DMU_FALSE},
  {DMU_VIDEO_SETUPPATH_FAILED, "Unable to setup video path",DMU_FALSE},
  {DMU_VIDEO_BAD_CONTROL_VALUE, "Failed to set control value",DMU_FALSE},
  {DMU_VIDEO_BUF_REGISTER_FAILED,
   "Unable to register buffer pool with the video path", DMU_FALSE},
  {DMU_VIDEO_START_FAILED, 
   "Couldn't start the video transfer", DMU_FALSE},
  {DMU_VIDEO_STARTED,
   "Video transfer already started!", DMU_FALSE},
  {DMU_VIDEO_NO_EVENT, 
   "No video event in queue", DMU_FALSE}, 
  {DMU_VIDEO_TO_DMBUFFER_FAILED, 
   "Unable to convert video event to dmbuffer", DMU_FALSE},
  {DMU_VIDEO_NO_PATH, 
   "No video path constructed.  Call setup() before attempting this call",
   DMU_FALSE},
  {VIDEOIN_TRANSFER_COMPLETE, 
   "(Temporary return value) Received a good transfer complete complete event",
   DMU_FALSE},
  {VIDEOIN_RANDOM_EVENT, 
   "(Temporary return value) Received a non-transfer complete event",
   DMU_FALSE},

  // DMIC errors
  {DMU_DMIC_NO_IMG_CTX_DEFINED,
   "No image conversion context is defined yet", DMU_FALSE},
  {DMU_DMIC_SRC_PARAMS_FAILED,
   "Failed to set the dmic params for the source pool", DMU_FALSE},
  {DMU_DMIC_DST_PARAMS_FAILED,
   "Failed to set the dmic params for the destination pool", DMU_FALSE},
  {DMU_DMIC_INVALID_QUALITY,
   "Quality value passed to object is invalid", DMU_FALSE},
  {DMU_DMIC_SEND_FAILED, "dmICSend failed", DMU_FALSE},
  {DMU_DMIC_RECEIVE_FAILED, "dmICReceive failed", DMU_FALSE},
  {DMU_DMIC_SET_CONV_FAILED, "dmICSetConvParams failed", DMU_FALSE},
  {DMU_DMIC_CREATE_FAILED, "dmICCreate failed", DMU_FALSE},
  {DMU_DMIC_FOUND_NO_USABLE_DMIC,
   "Was not able to find a dmic that matched parameters", DMU_FALSE},
  {DMU_DMIC_SRC_POOL_PARAMS_FAILED, "dmICGetSrcPoolParams failed", DMU_FALSE},
  {DMU_DMIC_DST_POOL_PARAMS_FAILED, "dmICGetDstPoolParams failed", DMU_FALSE},
  {DMU_DMIC_DST_POOL_FAILED,
   "failed to make new dmbuffer drain for image conversion ctx", DMU_FALSE},
  {DMU_DMIC_SET_IMAGE_DEFAULTS_FAILED, "dmSetImageDefault failed", DMU_FALSE},
  {DMU_DMIC_SET_POOL_DEFAULTS_FAILED, "dmSetPoolDefault failed", DMU_FALSE},
  {DMU_DMIC_PARAMS_SET_STRING_FAILED, "dmParamsSetString failed", DMU_FALSE},

  // GL Error messages
  {DMU_GL_ERROR, "Generic OpenGL error", DMU_FALSE},
  {DMU_GL_NO_FBCONFIG,
   "Can't get a frame buffer configuration which matches parameters",
   DMU_FALSE},
  {DMU_GL_PBUFFER_ALLOC_FAILED, "Failed to allocate pbuffer\n", DMU_FALSE },
  {DMU_GL_PBUFFER_SETUP_FAILED, "Failed to setup pbuffer\n", DMU_FALSE },
  {DMU_GL_PBUFFER_ASSOC_FAILED,
   "Failed to associate dmbuffer with GL pbuffer\n",DMU_FALSE},

  // Movie error messages
  {DMU_MOVIE_ERROR, "Generic Movie error", DMU_FALSE},
  {DMU_MOVIE_THREAD_ERROR, "Unable to start movie thread",DMU_FALSE},
  {DMU_MOVIE_FILE_ERROR, "Movie File I/O problem\n",DMU_FALSE},
  {DMU_MOVIE_SEMA_ERROR, "Unable to create or initialize semaphore lock",DMU_FALSE},
  {DMU_MOVIE_BAD_BUFFER_SIZE, "Requested buffer size too small",DMU_FALSE},
  {DMU_MOVIE_BAD_OFFSET,"Bad movie offset requested",DMU_FALSE},

  // Chain object error messages
  {DMU_CHAIN_ERROR, "Generic Chain error", DMU_FALSE},
  {DMU_CHAIN_NO_CHAIN,
   "Unable to perform method because there was no chain", DMU_FALSE},
  {DMU_CHAIN_USE_SET_FD, "Use SetFD instead of getFD", DMU_FALSE},

  // pixel transformation error message
  {DMU_PIXEL_ERROR, "Generic Pixel Transformation Error", DMU_FALSE},

  // network error messages
  {DMU_NETWORK_ERROR, "Generic Network Error", DMU_FALSE},
  NULL
};
// the number of error messages
const int numDmuErrors = (sizeof(dmuErrors) / sizeof(errorMsg));

#endif //_DMUERROR_H
