/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuGLCtx.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//

// <dmu>

// the dmuGLCtx is a subclass of dmuGLPbuffer that provides GLX
// context functionality around the PBuffer encapsulated in the
// dmuGLPBuffer class.  This class can be usefull for performing
// compositing sorts of operations on incoming video or movie data.
// At any point in time, the latest DMBuffer that has been passed in
// to the object can be <i>grab</i>'ed.  Once grabbed, the buffer is
// locked and safe to use.  Using calles to then makeCurrent,
// makeCurrentRead, and makeCurrentWrite methods will then allow you
// to perform rendering operations to and from this buffer (which
// already has data in it which has been passed down the dmu chain).
// Once a buffer is released, then the DMBuffer which had been grabed
// is then passed down the chain to the object's children - A display
// object, or a videoOut object, for example. <br><br>
// 
// Please note that this is a very useful object, but has not been 
// tested very well.  There are known problems with this class which 
// we are trying to track down for the next release of the library.

// </dmu>
#ifndef DMU_GLCTX_H
#define DMU_GLCTX_H

#include "dmuGLPBuffer.h"


class dmuGLCtx : public dmuGLPBuffer {
public:

  // Basic construction and parameter initialization
  dmuGLCtx();

  // Destroys the context (as well as the pbuffers and other structures 
  // that are associated with the dmuGLPbuffer parent object).
  ~dmuGLCtx();
  
  // This method performs the normal dmuGLPBuffer initialization, and 
  // then creates a glXContext to be used for rendering into and out
  // of the PBuffer.  
  int init(dmuParams *params = NULL);

  // <dmu> 
  // <B> Context methods </B> - The following methods call glx
  // functions to make the context for the PBuffer, and the current
  // associated dmbuffer, current for GL reading, and/or writing
  // operations.  See <i>glXMakeCurrent (3G) and glXMakeCurrentReadSGI
  // (3G) </i> for more details on how these work in GL.  Note that
  // <i>grab()</i> must be called on the object before performing any
  // of these operations, and <i> release </i> should be called as
  // soon as you are finished rendering into/out of the
  // pbuffer<br><UL> </dmu>
  int makeCurrentRead();
  int makeCurrentRead(GLXDrawable draw, GLXContext context);
  int makeCurrentRead(GLXDrawable draw);
  int makeCurrentWrite();
  int makeCurrent();
  
  // <dmu> </UL> </dmu>

protected:
  int createContext();
  
  GLXContext context;
};

#endif // DMU_GLCTX_H
