/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuIC.h
//
//  Author: Ed Allard <eda@esd.sgi.com> Tim Davison <td@pobox.com>
//  Date:   Oct, 1996
//
// <dmu>
//  This is a simple gl display object.  This object opens an openGL
//  window and does a drawpixels into that buffer of everything passed
//  down to it.
// </dmu>
//
#ifndef _DMUGLTEST_H
#define _DMUGLTEST_H

#include <GL/glx.h>
#include <GL/gl.h>
#include <dmedia/dm_buffer.h>
#include "ScratchWindow.h"
#include "dmuObj.h"

class dmuGLDisplay : public dmuObj {
public:
  dmuGLDisplay();
  ~dmuGLDisplay();

  // This method will actually do the creation of the window.  You must
  // set the image size and the zoom factor before calling this method.
  int setup(int interlace = DMU_FALSE);

  // This method  does nothing because this object does not add any
  // constraints to the dmbuffers 
  int addSrcConstraints(DMparams *params) { return DMU_OK; }
  // This method  does nothing because this object does not add any
  // constraints to the dmbuffers 
  int addDstConstraints(DMparams *params) { return DMU_OK; }

  // This method returns zero because it has no behavior to watch
  // a file descriptor for
  int getFD() { return 0; }

  // This object does not generate event to handle to this method
  // does nothing
  int handleEvents(fd_set fdSet) { return DMU_OK; }

  // Takes the data from the dmbuffer and maps it into memory as an
  // array.  Then it draws to the window using the dmbuffer as the
  // source.
  int receive(DMbuffer inBuffer);

  // set the type of pixel packing to draw using.  This should match
  // the pixel packing of the parent
  void setPacking(GLenum newPacking) { packing = newPacking; }

  // get the current pixel packing mode
  GLenum getPacking()                { return packing; }

  // <dmu>
  // <B> <LI>Set the image size</B> <UL>
  // if you do not use the constructor of the subclass which sets the
  // initail size, you can use this function to setup the width
  // of the image we are dealing with.  If you didn't use the constructor
  // to set the initail size, you must call this method  before the
  // setup method is called or the setup method will fail </dmu>
  void setImageSizeX(int newImageSizeX)  { imageSizeX = newImageSizeX; }
  void setImageSizeY(int newImageSizeY)  { imageSizeY = newImageSizeY; }
  void setImageDimensions(int newImageSizeX, int newImageSizeY) {
    imageSizeX = newImageSizeX;
    imageSizeY = newImageSizeY;
  }
  // <dmu> </UL> </dmu>

  // <dmu> <LI><B>Set the zoom factor</B><UL>
  // Set the zoom factor for the image.
  //</dmu>
  void setImageZoomX(int newZoomX) { zoomX = newZoomX; }
  void setImageZoomY(int newZoomY) { zoomY = newZoomY; }
  void setImageZoom(int newZoomX, int newZoomY) {
    zoomX = newZoomX;
    zoomY = newZoomY;
  }
  // <dmu> </UL> </dmu>
  
protected:
  int winSizeX;
  int winSizeY;
  int zoomX;
  int zoomY;
  Display      *xDisplay;    
  XVisualInfo  *xVisInfo;
  GLXContext    glxCtx;
  Window win;
  GLenum packing;

  ScratchWindow *window;

  virtual int easyToAdd() {return DMU_TRUE;};

  int field;
private:

};

#endif // _DMUGLTEST_H
