/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuGLTex2d.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// <dmu>
// <P>The dmuGLTex2d class inherits from dmuGLPBuffer, and
// implements the functionality needed to use the latest data that 
// has been receive from the parent object as a texture map in 
// subsequent OpenGL operations.  See the dmuGLPBuffer class for 
// more details on how dmuGLPBuffers work.</P>

// <P>The dmuGLTex2d object takes care of the <i>grab()</i> and
// <i>release()</i> method calls on the dmuGLPBuffer.  These are
// wrapped in the <i>defineTex2D</i> method.  The typical usage 
// scenario of dmuGLTex2d looks something like this: </P> <NL>

// <LI> Construct (and name) a dmuObject that will be used as the data
// source for the dmuGLTex2d object <BR><BR>

// <LI> Construct (and name) a dmuGLTex2d object<BR><BR>

// <LI> Wire the object into the dmu chain<BR><BR>

// <LI> Call <i>setup()</i> on the dmuGLTex2d object using parameters
// obtained from the application (theDisplay) and from the parent
// object (imageSizeX, imageSizeY) Make sure the image dimensions are
// powers of 2.<BR><BR>

// <LI> Start up the dmu chain (using start() at the head of the
// chain), and enter the normal handle events loop.  See the Overview
// for more information.<BR><BR>

// <LI> When the application wants to render a scene (after knowing
// that the parent of the dmuGLTex2d has gotten at least one valid
// frame of data), call <i>defineTex2D</i>.  This grabs the latest
// data, and sets up openGL to use the data as the current texture
// map.  (This data will be valid until the next call to
// <i>defineTex2D</i>.)<BR><BR>

// <LI> Render the scene.  Repeat the previous step whenever the
// application wants to render the scene (this often happens in a
// separate thread from the dmu handleEvents loop).<BR><BR>

// </NL> </dmu>
#ifndef DMUGLTEX2D_H
#define DMUGLTEX2D_H

#include "dmuObj.h"
#include "dmuGLPBuffer.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <assert.h>

#define DMU_GLTEX2D_NUM_FBPARAMS 8
#define DMU_GLTEX2D_FIRST_TEXID  1000

class dmuGLTex2d : public dmuGLPBuffer {
public:

  // Perform normal dmuGLPBuffer construction
  dmuGLTex2d();

  // Perform normal dmuGLPBuffer destruction 
  ~dmuGLTex2d();

  // This method <i>release()'s</i> any DMbuffer that it may have
  // <i>grab()'ed</i>.  It then calls <i>grab()</i> to lock and
  // associated the latest DMbuffer data and associate it with the
  // PBuffer.  Finally this method sets up the data in the PBuffer to
  // be used as the texture map source for all subsequent OGL
  // rendering operations (when texturing is enabled).  To achieve
  // this setup, it sets the current GLX readable to be the PBuffer,
  // and leaves the current GLXDrawable and context alone.  It then
  // makes a call to glCopyTexSubImage2DEXT which tells GL to use the
  // curent readable as the source for the texture map.  The texture
  // stays valid until a new one is defined, or until the user program
  // calls glXMakeCurrent or glXMakeCurrentReadSGI to set the context
  // and readable differently
  int defineTex2D();

  
protected:
  int gotPBuffer;
  unsigned int texID;

  // LastID is a static variable that provides us with free texture
  // ID's that we can use to bind textures.. this binding allows 
  // multiple texture objects in the sample OGL program
  static int nextFreeID;

  int texInitialized;
private:
};

#endif // DMUGLTEX2D_H
