/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuLatch.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// <dmu>
// This object allows you to grab a buffer and then manipulate is
// however you want.  Then you can send the buffer down the rest
// of the chain.  There are other classes that subclass from this one
// which associates gl contexts and pbuffer with the buffer past in.
// While you have a buffer in the grab()ed state, other buffer could
// come down the chain.  It will ignore all the buffers until you grab
// again.  When you grab, you always get the most current buffer that
// has come down the chain.  
//
// </dmu>
//
#ifndef DMU_LATCH_H
#define DMU_LATCH_H

// includes needed for this object.
#include "dmuObj.h"
#include "dmuUtil.h"

// system includes needed by this object
#include <ulocks.h>


class dmuLatch : public dmuObj {

public:

  // Constructs and initialization of member variables.This class is
  // intended as a base class, and should not be instantiated
  // directly.
  dmuLatch();
  
  // Destroys the object, and the PBuffer that is associated with 
  // the object.
  ~dmuLatch();

  // This method create the semaphores needed to create the latch
  int setup();

  // This method meerly calls init on it's parents
  virtual int init(dmuParams *params = NULL);

  // This method grabs the most recent DMBuffer that we've receive()'d
  // and locks it.  The locked buffer can then be used for other things.
  // The buffer remains valid and usable until we release() it.  While we
  // have a buffer grab()ed, other buffer can be coming down the chain,
  // but this object will ignore then until we grab() again.
  virtual int grab();

  // This unlocks the currently latched buffer so we can grab another one.
  // If the object current has a latched buffer, it will be propagated down
  // to the children.  This facilitates compositing operations.  
  virtual int release();


  // If this class is acting as a destination for another object in a
  // chain, this method is used to notify the parent object (i.e. the
  // source object) of any constraints which the video path may have
  // on its source.  These constraints are added to the <i>params</i>
  // parameter structure.  addSrcConstraints() is ususally called
  // internally in the library during the init() method of the parent
  // object.  It is rarely, if ever, used by user programs.
  virtual int addSrcConstraints(DMparams *params);

  // Similar to addSrcConstraints.  Our desination constraints are the same
  // as our parents destination constraints, so that's what we add.
  virtual int addDstConstraints(DMparams *params);

  // This method is used internal to the library.  It is called by the
  // parent object to notify the object that there is new data for it
  // to grab.
  virtual int receive(DMbuffer inBuffer);

  // This method returns a pointer to the data within the currently
  // latched buffer
  virtual unsigned char* mapBuffer();


  // Returns the USTMSC pairof the <i> grab()'ed </i> DMbuffer.  Returns
  // Undetermined results if the buffer has not been grabed.  Note
  // that some buffers may not have interesting MSCUST values, depending
  // on where they came from.
  virtual USTMSCpair getUSTMSC();

  // You can use this method to pass buffer right through the latch object
  virtual int handleEvents(fd_set fdSet);
  
  virtual int checkNewBuffer() {return (haveNewBuffer);}
protected:
  DMparams        *gfxParams;

  usema_t *newBufferLock;
  usema_t *latchBufferLock;

  int firstBufferIn;
  int haveNewBuffer;

  DMbuffer newBuffer;
  DMbuffer latchBuffer;

  // Convenience functions around the buffer locks
  void lockNewBuffer();
  void unlockNewBuffer();

  void lockLatchBuffer();
  void unlockLatchBuffer();
};

#endif
